// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:firebase_auth/firebase_auth.dart';
import 'package:wyatt_authentication_bloc/src/domain/entities/user.dart' as wyatt;

class UserFirebase implements wyatt.User {
  final User? _user;

  const UserFirebase(User user) : _user = user;

  User? get inner => _user;

  @override
  const UserFirebase.empty() : _user = null;

  @override
  DateTime? get creationTime => _user?.metadata.creationTime;

  @override
  String? get displayName => _user?.displayName;

  @override
  String? get email => _user?.email;

  @override
  bool get emailVerified => _user?.emailVerified ?? false;

  @override
  bool get isAnonymous => _user?.isAnonymous ?? false;

  @override
  bool get isEmpty => _user == null;

  @override
  bool get isNotEmpty => _user != null;

  @override
  DateTime? get lastSignInTime => _user?.metadata.lastSignInTime;

  @override
  String? get phoneNumber => _user?.phoneNumber;

  @override
  String? get photoURL => _user?.photoURL;

  @override
  String? get refreshToken => _user?.refreshToken;

  @override
  String get uid => _user?.uid ?? '';

  @override
  String? get providerId => _user?.providerData.first.providerId;

  @override
  bool? get isNewUser {
    if (_user?.metadata.lastSignInTime == null ||
        _user?.metadata.creationTime == null) {
      return null;
    } else {
      return _user?.metadata.lastSignInTime == _user?.metadata.creationTime;
    }
  }

  @override
  // ignore: lines_longer_than_80_chars
  String toString() => 'UserFirebase(creationTime: $creationTime, displayName: $displayName, email: $email, emailVerified: $emailVerified, isAnonymous: $isAnonymous, lastSignInTime: $lastSignInTime, phoneNumber: $phoneNumber, photoURL: $photoURL, refreshToken: $refreshToken, uid: $uid)';
}
